import { SelectedElement, SuperElementBuildProps } from './protocols/index.js';
declare type ElementRootReference = HTMLElement | (ShadowRoot & {
    adoptedStyleSheets: CSSStyleSheet[];
});
export declare class SuperElement extends HTMLElement {
    /**
     * The root of the element depends if its has or
     * not a shadowDOM. If has, it returns the shadowRoot
     * when not it returns the element itself.
     *
     * @returns reference to the current root of the element.
     */
    get root(): ElementRootReference;
    /**
     * Options to configure the element building. (Optional)
     */
    constructor({ mode, shadowRoot }?: SuperElementBuildProps);
    /**
     * Method called in element instantiation
     * and after `render` and `styling`.
     */
    init(): void;
    /**
     * Default lifecycle method of Web Components.
     * It runs when the element enters in the DOM.
     */
    connectedCallback(): void;
    /**
     * Default lifecycle method of Web Components.
     * It runs when the element is removed from DOM.
     */
    disconnectedCallback(): void;
    /**
     * Default lifecycle method of Web Components.
     * It runs when some observed attributes of the
     * element changes (add, change, delete).
     *
     * @param name Attribute name.
     * @param oldValue The old value of the attribute.
     * @param newValue The new value its receiving.
     *
     * This method requires that you define which are
     * the observed attributes. To define it, register
     * a static get method called observedAttributes
     * that returns the attributes names in an array.
     * ```ts
     * static get observedAttributes() {
     *   return ['attr']
     * }
     * ```
     */
    attributeChangedCallback(_name: string, _oldValue: unknown, _newValue: unknown): void;
    /**
     * Default lifecycle method of Web Components.
     * It runs when the element is moved to another
     * document. Using `document.adoptNode` for example.
     */
    adoptedCallback(): void;
    /**
     * Equavalent to addEventListener method.
     *
     * @param type Event type name. Like 'click', 'change', 'mouseover', etc...
     * @param listener A listener for the event.
     *  It can be a `EventListener` or an `EventListenerObject`.
     * @param options A list of configurations for the event listener.
     */
    on(type: keyof HTMLElementEventMap | string, listener: EventListenerOrEventListenerObject, options?: AddEventListenerOptions): void;
    /**
     * Equavalent to addEventListener method,
     * but runs only once.
     *
     * This method always has
     * the `once` option of AddEventListenerOptions as `true`.
     *
     * @param type Event type name. Like 'click', 'change', 'mouseover', etc...
     * @param listener A listener for the event.
     *  It can be a `EventListener` or an `EventListenerObject`.
     * @param options A list of configurations for the event listener.
     */
    once(type: keyof HTMLElementEventMap | string, listener: EventListenerOrEventListenerObject, options?: AddEventListenerOptions): void;
    /**
     * Selects an element by its query.
     *
     * Works using `element.root.querySelector` function and
     * aditionally add the `on`, `once`, `select` and `selectAll`
     * functions to the result element.
     *
     * @param query An element search query, like CSS selection queries.
     */
    select<T extends HTMLElement>(query: string): SelectedElement<T> | undefined;
    /**
     * Selects all elements that matches a query.
     *
     * Works using `element.root.querySelectorAll` function and
     * aditionally add the `on`, `once`, `select` and `selectAll`
     * functions to the result elements.
     *
     * @param query An element search query, like CSS selection queries.
     */
    selectAll<T extends HTMLElement>(query: string): SelectedElement<T>[];
    /**
     * This method defines the initial CSS of the
     * element, it runs on elements instatiation.
     *
     * @example
     * ```ts
     * styling() {
     *   return `
     *     div { ... }
     *   `
     * }
     * ```
     */
    styling(): string;
    /**
     * This method defines the initial HTML of the
     * element, it runs on elements instantiation.
     *
     * @example
     * ```ts
     *  render() {
     *    return `
     *      <div>...</div>
     *    `
     *  }
     * ```
     */
    render(): string | HTMLElement;
}
export {};
//# sourceMappingURL=super-element.d.ts.map